/******************************************************************
 * Copyright and Permission Notice                                *
 *                                                                *
 * Permission is granted to copy, use, modify, and merge this     *
 * software into your applications and to permit others to do any *
 * of the foregoing for non-commercial use only. You must include *
 * this permission and copyright notice in all copies and         *
 * modified versions of this software, and include attribution    *
 * to Taligent in all splash screens included in any application  *
 * using this software. You must require others to whom you       *
 * distribute this software to do likewise.                       *
 *                                                                *
 * To obtain an unrestricted commercial license to the software   *
 * source and/or binaries, please contact Taligent at             *
 * widgets@taligent.com                                           *
 *                                                                *
 * THE SOFTWARE IS PROVIDED IN ITS 'AS IS' CONDITION FOR          *
 * NON-COMMERCIAL USE ONLY. TALIGENT DISCLAIMS ANY LIABILITY OF   *
 * ANY KIND FOR DAMAGES WHATSOEVER RESULTING FROM THE USE OF      *
 * THIS SOFTWARE.                                                 *
 *                                                                *
 * (C) Copyright Taligent, Inc. 1996,1997-All Rights Reserved.    *
 * (C) Copyright IBM Corporation 1996,1997                        *
 ******************************************************************/

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import COM.taligent.util.*;
import COM.taligent.util.event.*;
import COM.taligent.widget.*;
import COM.taligent.widget.event.*;

/**
 * Basic sample applet to demonstrate the various features of
 * the MultiColumnListbox.
 *
 * @author  Andy Clark, Taligent Inc.
 * @version 1.0 (JDK 1.1)
 */
public class MultiColumnListboxApplet
    extends Applet
    implements WindowListener,
               ActionListener, ItemListener,
               ListboxListener, CaptionBarListener
    {
    //
    // Constants
    //

    /** This class. */
    private static final Class THIS = MultiColumnListboxApplet.class;

    /** Resource bundle containing BeanInfo resources used for L10N. */
    private static ResourceBundle RESOURCES;

    /** Name keys. */
    private static final String NAME_KEYS[] = {
        "name1", "name2", "name3", "name4",
        "name5", "name6", "name7", "name8"
        };


    //
    // GUI Components
    //
    private BorderPanel   fAddData        = new BorderPanel(BorderPanelStyle.IN);
    private Checkbox      fAllowResize    = new Checkbox();
    private Checkbox      fAllowSort      = new Checkbox();
    private CheckboxGroup fSortTypes      = new CheckboxGroup();
    private Checkbox      fAlphabeticSort = new Checkbox();
    private Checkbox      fNumericSort    = new Checkbox();
    private Button        fAddColumn      = new Button();

    private BorderPanel fSettings               = new BorderPanel(BorderPanelStyle.IN);
    private Checkbox    fAllowMultipleSelection = new Checkbox();
    private Checkbox    fShowHSeparator         = new Checkbox();
    private Checkbox    fShowVSeparator         = new Checkbox();
    private Button      fListboxSettings        = new Button();
    private Button      fSelectedColumn         = new Button();

    private BorderPanel        fExample = new BorderPanel(BorderPanelStyle.IN);
    private MultiColumnListbox fListbox = new MultiColumnListbox();

    private Label fInfo = new Label();

    private RowColorDialog fListboxSettingsDialog = new RowColorDialog(this, fListbox);
    private ColumnDialog   fSelectedColumnDialog  = new ColumnDialog(this, fListbox);

    //
    // Data
    //
    private int fColumns = 0;

    /** Main. */
    public static void main(String argv[]) {

        // create applet
        Applet applet = new MultiColumnListboxApplet();
        applet.init();

        // create containing frame
        Frame frame = new Frame(applet.getClass().getName());
        frame.setLayout(new BorderLayout());
        frame.add("Center", applet);

        // show
        frame.setSize(500, 400);
        frame.setVisible(true);
        }

    /** Localizes all of the strings in the applet. */
    public void setLocale(Locale locale) {

        // reload resources
        loadResources(locale);

        // add data
        fAddData.setText(getString("addData"));
        fAllowResize.setLabel(getString("allowResize"));
        fAllowSort.setLabel(getString("allowSort"));
        fAlphabeticSort.setLabel(getString("alphabetic"));
        fNumericSort.setLabel(getString("numeric"));
        fAddColumn.setLabel(getString("addColumn"));

        // settings
        fSettings.setText(getString("settings"));
        fAllowMultipleSelection.setLabel(getString("allowMultiple"));
        fShowHSeparator.setLabel(getString("showHSeparator"));
        fShowVSeparator.setLabel(getString("showVSeparator"));
        fListboxSettings.setLabel(getString("listboxSettings"));
        fSelectedColumn.setLabel(getString("selectedColumnButton"));

        // example
        fExample.setText(getString("example"));

        // reset display
        validate();

        // now allow the locale to change
        super.setLocale(locale);

        // set locale information for dialogs
        fListboxSettingsDialog.setLocale(locale);
        fSelectedColumnDialog.setLocale(locale);
        }

    /** Validates and repaints the display. */
    public void validate() {

        super.validate();
        repaint();
        }

    //
    // L10N utility methods
    //

    /** Loads the resources for this Object. */
    private static void loadResources() {

        loadResources(Locale.getDefault());
        }

    /** Loads the resources for this Object. */
    private static void loadResources(Locale locale) {

        try {
            RESOURCES = ResourceBundle.getBundle(
                            THIS.getName() + "Resources",
                            locale
                            );
            }
        catch (MissingResourceException e) {
            // do nothing
            }
        }

    /**
     * Returns the string value of the specified key. If the value
     * of the specified key is not found, the key is returned as the
     * value.
     *
     * @param key   The key of the value to return.
     */
    protected static String getString(String key) {

        return getString(key, key);
        }

    /**
     * Returns the string value of the specified key. If the resource
     * bundle is not found or the key is not contained in the resource
     * bundle, the default value will be returned.
     *
     * @param key   The key of the value to return
     * @param value The default value to return if the real value of
     *              the specified key is not found.
     */
    protected static String getString(String key, String value) {

        if (RESOURCES != null) {
            try {
                return RESOURCES.getString(key);
                }
            catch (MissingResourceException e) {
                // return default value
                }
            }

        return value;
        }

    //
    // Main program
    //

    /** Initialize the contents of the applet. */
    public void init() {

        // localize components
        setLocale(Locale.getDefault());

        // setup applet
        setBackground(Color.lightGray);
        setLayout(new BorderLayout());

        // Outer border
        BorderPanel outerborder = new BorderPanel(Color.lightGray, 4);
        outerborder.setGap(0);
        outerborder.setLayout(new BorderLayout());

        // Inner border
        BorderPanel border = new BorderPanel(BorderPanel.RAISED, 2);
        border.setLayout(new BorderLayout());

        // Main panel
        Panel panel = new Panel();
        panel.setLayout(new BorderLayout());

        // Upper main panel
        Panel upper = new Panel();
        upper.setLayout(new GridLayout(1, 2));

        // Setup "add data" panel
        fAddData.setLayout(new BorderLayout());

        Panel addparams = new Panel();
        fSortTypes = new CheckboxGroup();
        fAlphabeticSort.setCheckboxGroup(fSortTypes);
        fNumericSort.setCheckboxGroup(fSortTypes);
        fSortTypes.setCurrent(fAlphabeticSort);
        fAlphabeticSort.setEnabled(false);
        fNumericSort.setEnabled(false);

        addparams.setLayout(new GridLayout(4, 1, 2, 2));
        fAllowResize.setState(true);
        addparams.add(fAllowResize);
        addparams.add(fAllowSort);
        addparams.add(fAlphabeticSort);
        addparams.add(fNumericSort);

        fAddData.add("Center", addparams);
        fAddData.add("South", fAddColumn);

        upper.add(fAddData);

        // Setup "settings" panel
        fSettings.setLayout(new GridLayout(5, 1, 2, 2));
        fSettings.add(fAllowMultipleSelection);
        fSettings.add(fShowHSeparator);
        fSettings.add(fShowVSeparator);
        fSettings.add(fListboxSettings);
        fSelectedColumn.setEnabled(false);
        fSettings.add(fSelectedColumn);

        upper.add(fSettings);

        panel.add("North", upper);

        // setup "example" panel
        fExample.setLayout(new BorderLayout());
        fListbox.setContinuousDrag(true);
        /// fListbox.setHorizontalScrollbarVisible(true);

        fExample.add("Center", fListbox);

        panel.add("Center", fExample);

        border.add("Center", panel);

        // setup status bar
        BorderPanel insetborder = new BorderPanel(BorderPanelStyle.LOWERED, 1);
        insetborder.setGap(0);
        insetborder.setLayout(new BorderLayout());
        insetborder.setFont(new Font("TimesRoman", Font.PLAIN, 12));
        insetborder.add("Center", fInfo);

        border.add("South", insetborder);

        outerborder.add("Center", border);

        add("Center", outerborder);

        // add listeners
        AWT.getFrame(this).addWindowListener(this);
        fAddColumn.addActionListener(this);
        fAllowResize.addItemListener(this);
        fAllowSort.addItemListener(this);
        fAlphabeticSort.addItemListener(this);
        fNumericSort.addItemListener(this);

        fListbox.addCaptionBarListener(this);
        fListbox.addListboxListener(this);

        fAllowMultipleSelection.addItemListener(this);
        fShowHSeparator.addItemListener(this);
        fShowVSeparator.addItemListener(this);
        fSelectedColumn.addActionListener(this);
        fListboxSettings.addActionListener(this);

        // resize and show yourself
        setSize(500, 400);
        setVisible(true);
        }

    /** Convert modifiers to string representation. */
    public static String showModifiers(ListboxEvent evt) {
        StringBuffer str = new StringBuffer();

        if (evt.getModifiers() != 0)
            str.append(", "+getString("modifiers")+": ");

        if (evt.isAltDown())
            str.append("ALT ");
        if (evt.isControlDown())
            str.append("CTRL ");
        if (evt.isMetaDown())
            str.append("META ");
        if (evt.isShiftDown())
            str.append("SHIFT ");

        return str.toString();
        }

    /** Convert click count to string representation. */
    public static String showClicks(int count) {

        return (count > 1) ? ", "+getString("clicks")+": "+count : "";
        }

    /** Window event handlers. */
    public void windowOpened(WindowEvent evt) {}
    public void windowClosed(WindowEvent evt) {}
    public void windowIconified(WindowEvent evt) {}
    public void windowDeiconified(WindowEvent evt) {}
    public void windowActivated(WindowEvent evt) {}
    public void windowDeactivated(WindowEvent evt) {}
    public void windowClosing(WindowEvent evt) {

        System.exit(0);
        }

    /** Button was pressed. */
    public void actionPerformed(ActionEvent evt) {
        Object source = evt.getSource();

        // add a column
        if (source == fAddColumn) {
            // add column
            ListboxColumn column = fListbox.addColumn(getString("column")+" "+fColumns);
            column.setWidth(80);
            column.setResizable(fAllowResize.getState());

            // add rows of data
            fColumns++;
            if (fColumns == 1) {
                fListbox.setUpdate(false);
                for (int i = 0; i < 20; i++)
                    fListbox.addRow(new String[0]);
                fListbox.setUpdate(true);
                }

            // add data
            if (fAllowSort.getState()) {
                //column.setSorter(new SelectionSorter());
                column.setSorter(new HeapSorter());
                if (fSortTypes.getCurrent() == fAlphabeticSort)
                    column.getSorter().setComparator(new StringComparator());
                else {
                    column.setAlignment(Alignment.RIGHT);
                    column.getSorter().setComparator(new NumericStringComparator());
                    }
                }
            else
                column.setSorter(null);

            // add a new column of data
            String col[] = new String[20];
            boolean alphabetic = fSortTypes.getCurrent() == fAlphabeticSort;
            boolean nosort = !fAllowSort.getState();
            for (int i = 0; i < 20; i++) {
                if (alphabetic || nosort)
                    col[i] = getString(NAME_KEYS[i % NAME_KEYS.length]);
                else
                    col[i] = Integer.toString(i);
                }
            fListbox.replaceColumn(col, fColumns - 1);

            fSelectedColumn.setEnabled(true);
            }

        // Set colors
        else if (source == fListboxSettings)
            fListboxSettingsDialog.setVisible(true);
        else if (source == fSelectedColumn)
            fSelectedColumnDialog.setVisible(true);
        }

    /** A checkbox was clicked. */
    public void itemStateChanged(ItemEvent evt) {
        Object source = evt.getSource();

        if (source instanceof Checkbox) {
            // Allow sorting
            if (source == fAllowSort) {
                if (fAllowSort.getState()) {
                    fAlphabeticSort.setEnabled(true);
                    fNumericSort.setEnabled(true);
                    }
                else {
                    fAlphabeticSort.setEnabled(false);
                    fNumericSort.setEnabled(false);
                    }
                }

            // Allow/Disallow multiple selections
            else if (source == fAllowMultipleSelection) {
                boolean multiple = fAllowMultipleSelection.getState();
                fListbox.setMultipleSelections(multiple);
                }

            // Display various separators
            else if (source == fShowHSeparator) {
                boolean visible = fShowHSeparator.getState();
                fListbox.setHorizontalSeparatorVisible(visible);
                }
            else if (source == fShowVSeparator) {
                boolean visible = fShowVSeparator.getState();
                fListbox.setVerticalSeparatorVisible(visible);
                }
            }
        }

    /** A row was selected. */
    public void rowSelected(ListboxEvent evt) {

        fInfo.setText(getString("selectedRow")+" "
                        +evt.getRow()
                        +showModifiers(evt)
                        +showClicks(evt.getClickCount())
                        );
        }

    /** A row was deselected. */
    public void rowDeselected(ListboxEvent evt) {

        fInfo.setText(getString("deselectedRow")+" "
                        +evt.getRow()
                        +showModifiers(evt)
                        +showClicks(evt.getClickCount())
                        );
        }

    /** A caption was selected. */
    public void captionSelected(CaptionBarEvent evt) {

        fInfo.setText(getString("selectedColumn")+" "
                        +evt.getCaption()
                        +showClicks(evt.getClickCount())
                        );
        }

    /** A caption was resized. */
    public void captionResized(CaptionBarEvent evt) {

        fInfo.setText(getString("resizedColumn")+" "
                        +evt.getCaption()
                        +showClicks(evt.getClickCount())
                        );
        }
    }

/**
 * Basic settings dialog box.
 * <p>
 * Again, problems with Dialog boxes persist... ARGH! So I "fake"
 * a dialog box by having the parent's components disabled
 * recursively by calling setEnabled(Component, boolean). -Ac
 */
class SettingsDialog
    extends Frame
    implements WindowListener
    {
    //
    // Constants
    //
    /** Color key strings. */
    protected static final String COLOR_KEYS[] = {
        "black", "blue", "cyan", "darkGray", "gray",
        "green", "lightGray", "magenta", "orange",
        "pink", "red", "white", "yellow"
        };

    /** Color values. */
    protected static final Color COLOR_VALUES[] = {
        Color.black, Color.blue, Color.cyan, Color.darkGray,
        Color.gray, Color.green, Color.lightGray,
        Color.magenta, Color.orange, Color.pink, Color.red,
        Color.white, Color.yellow
        };

    /** Constructor. */
    public SettingsDialog() {

        addWindowListener(this);
        }

    /** Returns the MultiColumnListboxApplet getString() value. */
    public static String getString(String key) {

        return MultiColumnListboxApplet.getString(key);
        }

    /** Validates and repaints the display. */
    public void validate() {

        super.validate();
        repaint();
        }

    /** Converts color string to Color. */
    public Color stringToColor(String string) {

        for (int i = 0; i < COLOR_KEYS.length; i++)
            if (MultiColumnListboxApplet.getString(COLOR_KEYS[i]).equals(string))
                return COLOR_VALUES[i];

        return null;
        }

    /** Converts Color to color string. */
    public String colorToString(Color color) {

        for (int i = 0; i < COLOR_VALUES.length; i++)
            if (COLOR_VALUES[i].equals(color))
                return MultiColumnListboxApplet.getString(COLOR_KEYS[i]);

        return null;
        }

    /** Dis/enables parent's components, recursively. */
    public static void setEnabled(Component comp, boolean enabled) {

        comp.setEnabled(enabled);
        if (comp instanceof Container) {
            Component comps[] = ((Container)comp).getComponents();
            for (int i = 0; i < comps.length; i++)
                setEnabled(comps[i], enabled);
            }
        }

    /** Window event handlers. */
    public void windowOpened(WindowEvent evt) {}
    public void windowClosed(WindowEvent evt) {}
    public void windowIconified(WindowEvent evt) {}
    public void windowDeiconified(WindowEvent evt) {}
    public void windowActivated(WindowEvent evt) {}
    public void windowDeactivated(WindowEvent evt) {}
    public void windowClosing(WindowEvent evt) {

        setVisible(false);
        }
    }

/**
 * Column color selection and various attributes dialog.
 */
class ColumnDialog
    extends SettingsDialog
    implements ActionListener
    {
    //
    // GUI Components
    //
    BorderPanel fSettings        = new BorderPanel(BorderPanelStyle.IN);
    Label       fForegroundLabel = new Label();
    Choice      fForeground      = new Choice();
    Label       fBackgroundLabel = new Label();
    Choice      fBackground      = new Choice();
    Label       fResizableLabel  = new Label();
    Choice      fResizable       = new Choice();
    Label       fVisibleLabel    = new Label();
    Choice      fVisible         = new Choice();
    Label       fAlignmentLabel  = new Label();
    Choice      fAlignment       = new Choice();

    Button fOk     = new Button();
    Button fCancel = new Button();

    //
    // Data
    //
    Component          fParent;
    MultiColumnListbox fListbox;
    ListboxColumn      fColumn;

    /** Localizes all of the strings in the applet. */
    public void setLocale(Locale locale) {

        // settings
        fSettings.setText(getString("columnSettings"));

        fForegroundLabel.setText(getString("foreground"));
        fBackgroundLabel.setText(getString("background"));

        fForeground.removeAll();
        fBackground.removeAll();

        fForeground.addItem(getString("default"));
        fBackground.addItem(getString("default"));

        for (int i = 0; i < COLOR_KEYS.length; i++) {
            String color = getString(COLOR_KEYS[i]);
            fForeground.addItem(color);
            fBackground.addItem(color);
            }

        fResizableLabel.setText(getString("resizable"));
        fVisibleLabel.setText(getString("visible"));
        fAlignmentLabel.setText(getString("alignment"));

        // buttons
        fOk.setLabel(getString("ok"));
        fCancel.setLabel(getString("cancel"));

        // reset display
        validate();

        // now allow the locale to change
        super.setLocale(locale);
        }

    /** Constructor. */
    public ColumnDialog(Component parent, MultiColumnListbox listbox) {

        // save references
        fParent  = parent;
        fListbox = listbox;

        setBackground(Color.lightGray);
        //setLayout(new BorderLayout()); // this is default layout anyway

        // setup column settings
        fSettings.setAlignment(Alignment.CENTER);
        fSettings.setLayout(new GridLayout(5, 2, 2, 2));

        fSettings.add(fForegroundLabel);
        fSettings.add(fForeground);
        fSettings.add(fBackgroundLabel);
        fSettings.add(fBackground);

        fSettings.add(fResizableLabel);
        fResizable.addItem(Boolean.TRUE.toString());
        fResizable.addItem(Boolean.FALSE.toString());
        fSettings.add(fResizable);

        fSettings.add(fVisibleLabel);
        fVisible.addItem(Boolean.TRUE.toString());
        fVisible.addItem(Boolean.FALSE.toString());
        fSettings.add(fVisible);

        fSettings.add(fAlignmentLabel);
        for (int i = 0; i < Alignment.TAGS.length; i++)
            fAlignment.addItem(Alignment.TAGS[i]);
        fSettings.add(fAlignment);

        add("Center", fSettings);

        // setup buttons
        Panel buttonpanel = new Panel();
        //buttonpanel.setLayout(new FlowLayout()); // this is default anyway
        buttonpanel.add(fOk);
        buttonpanel.add(fCancel);

        add("South", buttonpanel);

        // add listeners
        fOk.addActionListener(this);
        fCancel.addActionListener(this);

        resize(220, 200);
        }

    /** Override colorToString. */
    public String colorToString(Color color) {
        String string = super.colorToString(color);
        if (string == null)
            string = getString("default");

        return string;
        }

    /** Each time the dialog is shown, initialize the settings. */
    public void setVisible(boolean show) {

        if (show) {
            // find selected column
            int col = 0;
            for (int i = 0; i < fListbox.getColumnCount(); i++)
                if (fListbox.getColumnInfo(i).isSelected()) {
                    fColumn = fListbox.getColumnInfo(i);
                    col = i;
                    break;
                    }

            // set title
            setTitle(getString("column")+' '+col);

            // set the column colors
            fBackground.select(colorToString(fColumn.getBackground()));
            fForeground.select(colorToString(fColumn.getForeground()));

            // set resizable and shown flags
            Boolean resizable = fColumn.isResizable() ? Boolean.TRUE : Boolean.FALSE;
            fResizable.select(resizable.toString());
            Boolean visible = fColumn.isVisible() ? Boolean.TRUE : Boolean.FALSE;
            fVisible.select(visible.toString());

            // set alignment
            fAlignment.select(fColumn.getAlignment().toString());
            }

        setEnabled(fParent, !show);
        super.setVisible(show);
        }

    /** Apply settings to listbox. */
    public void applySettings() {

        // turn off updates
        boolean oldupdate = fListbox.getUpdate();
        fListbox.setUpdate(false);

        // set column settings
        Color color = stringToColor(fForeground.getSelectedItem());
        fColumn.setForeground(color);
        fColumn.setBackground(stringToColor(fBackground.getSelectedItem()));

        // set resizable and shown flags
        fColumn.setResizable(fResizable.getSelectedIndex() == 0);
        fColumn.setVisible(fVisible.getSelectedIndex() == 0);

        // set alignment
        switch (fAlignment.getSelectedIndex()) {
            case 0: fColumn.setAlignment(Alignment.LEFT);   break;
            case 1: fColumn.setAlignment(Alignment.CENTER); break;
            case 2: fColumn.setAlignment(Alignment.RIGHT);  break;
            }

        // turn on updates
        fListbox.setUpdate(oldupdate);
        }

    /** Button pressed. */
    public void actionPerformed(ActionEvent evt) {
        Object source = evt.getSource();

        if (source == fOk)
            applySettings();

        setVisible(false);
        }
    }

/**
 * Row color selection dialog.
 */
class RowColorDialog
    extends SettingsDialog
    implements ActionListener
    {
    //
    // Constants
    //
    private static final String IMAGE_NAMES[] = {
        null, "background.jpg", "me.gif", "rainbow.jpg", "chain.gif"
        };
    private static final Image  IMAGES[] = new Image[IMAGE_NAMES.length];

    //
    // GUI Components
    //
    private BorderPanel fSettings = new BorderPanel(BorderPanelStyle.IN);
    private Label       fForegroundLabel = new Label();
    private Choice      fForeground      = new Choice();
    private Label       fBackgroundLabel = new Label();
    private Choice      fBackground      = new Choice();
    private Label       fSelectionForegroundLabel = new Label();
    private Choice      fSelectionForeground      = new Choice();
    private Label       fSelectionBackgroundLabel = new Label();
    private Choice      fSelectionBackground      = new Choice();
    private Label       fBackgroundImageLabel           = new Label();
    private Choice      fBackgroundImage                = new Choice();
    private Label       fCaptionBarBackgroundImageLabel = new Label();
    private Choice      fCaptionBarBackgroundImage      = new Choice();

    private Button fOk     = new Button();
    private Button fCancel = new Button();

    //
    // Data
    //
    private Component          fParent;
    private MultiColumnListbox fListbox;

    /** Localizes all of the strings in the applet. */
    public void setLocale(Locale locale) {

        // title
        setTitle(getString("listboxSettings"));

        // settings
        fSettings.setText(getTitle());

        fForegroundLabel.setText(getString("rowForeground"));
        fBackgroundLabel.setText(getString("rowBackground"));
        fSelectionForegroundLabel.setText(getString("selectionForeground"));
        fSelectionBackgroundLabel.setText(getString("selectionBackground"));

        fForeground.removeAll();
        fBackground.removeAll();
        fSelectionForeground.removeAll();
        fSelectionBackground.removeAll();

        for (int i = 0; i < COLOR_KEYS.length; i++) {
            String color = getString(COLOR_KEYS[i]);
            fForeground.addItem(color);
            fBackground.addItem(color);
            fSelectionForeground.addItem(color);
            fSelectionBackground.addItem(color);
            }

        fBackgroundImageLabel.setText(getString("backgroundImage"));
        fCaptionBarBackgroundImageLabel.setText(getString("captionbarImage"));

        if (fBackgroundImage.getItemCount() > 0) {
            fBackgroundImage.remove(0);
            fCaptionBarBackgroundImage.remove(0);
            fBackgroundImage.insert(getString("none"), 0);
            fCaptionBarBackgroundImage.insert(getString("none"), 0);
            }

        // buttons
        fOk.setLabel(getString("ok"));
        fCancel.setLabel(getString("cancel"));

        // reset display
        validate();

        // now allow the locale to change
        super.setLocale(locale);
        }

    /** Constructor. */
    public RowColorDialog(Component parent, MultiColumnListbox listbox) {

        // save references
        fParent  = parent;
        fListbox = listbox;

        setBackground(Color.lightGray);
        //setLayout(new BorderLayout()); // this is default layout anyway

        // setup listbox settings
        fSettings.setAlignment(Alignment.CENTER);
        fSettings.setLayout(new GridLayout(6, 2, 2, 2));

        // init image choices
        fBackgroundImage.addItem("none");
        fCaptionBarBackgroundImage.addItem("none");
        for (int i = 1; i < IMAGE_NAMES.length; i++) {
            fBackgroundImage.addItem(IMAGE_NAMES[i]);
            fCaptionBarBackgroundImage.addItem(IMAGE_NAMES[i]);
            }

        fSettings.add(fForegroundLabel);
        fSettings.add(fForeground);
        fSettings.add(fBackgroundLabel);
        fSettings.add(fBackground);
        fSettings.add(fSelectionForegroundLabel);
        fSettings.add(fSelectionForeground);
        fSettings.add(fSelectionBackgroundLabel);
        fSettings.add(fSelectionBackground);
        fSettings.add(fBackgroundImageLabel);
        fSettings.add(fBackgroundImage);
        fSettings.add(fCaptionBarBackgroundImageLabel);
        fSettings.add(fCaptionBarBackgroundImage);

        add("Center", fSettings);

        // setup buttons
        Panel buttonpanel = new Panel();
        //buttonpanel.setLayout(new FlowLayout()); // this is default anyway
        buttonpanel.add(fOk);
        buttonpanel.add(fCancel);

        // add listeners
        fOk.addActionListener(this);
        fCancel.addActionListener(this);

        add("South", buttonpanel);

        resize(350, 250);

        try {
            MediaTracker tracker = new MediaTracker(this);

            for (int i = 1; i < IMAGES.length; i++) {
                IMAGES[i] = Toolkit.getDefaultToolkit().getImage(IMAGE_NAMES[i]);
                tracker.addImage(IMAGES[i], i);
                }
            tracker.waitForAll();
            }
        catch (Exception e) {
            }
        }

    /** Each time the dialog is shown, initialize the settings. */
    public void setVisible(boolean visible) {

        if (visible) {
            // initialize contents of boxes
            fForeground.select(colorToString(fListbox.getForeground()));
            fBackground.select(colorToString(fListbox.getBackground()));
            fSelectionForeground.select(colorToString(fListbox.getSelectionForeground()));
            fSelectionBackground.select(colorToString(fListbox.getSelectionBackground()));

            // set background image
            Image image1 = fListbox.getBackgroundImage();
            Image image2 = fListbox.getCaptionBarBackgroundImage();
            for (int i = 0; i < IMAGES.length; i++) {
                if (image1 == IMAGES[i])
                    fBackgroundImage.select(i);
                if (image2 == IMAGES[i])
                    fCaptionBarBackgroundImage.select(i);
                }
            }

        setEnabled(fParent, !visible);
        super.setVisible(visible);
        }

    /** Apply settings to listbox. */
    public void applySettings() {

        boolean oldupdate = fListbox.getUpdate();
        fListbox.setUpdate(false);

        // set listbox settings
        fListbox.setForeground(stringToColor(fForeground.getSelectedItem()));
        fListbox.setBackground(stringToColor(fBackground.getSelectedItem()));
        fListbox.setSelectionForeground(stringToColor(fSelectionForeground.getSelectedItem()));
        fListbox.setSelectionBackground(stringToColor(fSelectionBackground.getSelectedItem()));
        fListbox.setBackgroundImage(IMAGES[fBackgroundImage.getSelectedIndex()]);
        fListbox.setCaptionBarBackgroundImage(IMAGES[fCaptionBarBackgroundImage.getSelectedIndex()]);

        fListbox.setUpdate(oldupdate);
        }

    /** Button pressed. */
    public void actionPerformed(ActionEvent evt) {
        Object source = evt.getSource();

        if (source == fOk)
            applySettings();

        setVisible(false);
        }
    }
